/* 
  Copyright (C) 1998 Christofer Edvardsen and Norbert Preining

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "smmgr.h"

int repl_imon(const char *s) {
	char tmpstr[256];
	if (!device_initialized) {
		logit(L_INFO,"Device not initialize - calling initdevice!\n");
		repl_initdevice(NULL);
	}
	setiospeed(B38400);
	/* reset clock but do NOT write libfile */
	sendstring("AT+MCS=1", NULL,NULL);
	resetclock(0);
	sprintf(tmpstr,"AT+MCF=%d+MCV=%d+MCM=%d+MCD=%d+MCR=%d&W",
		s_fax,s_voice,s_monitor,s_remote,s_answer);
	sendstring(tmpstr, NULL,NULL);
	if (modemtypes[s_modemtype].id == USR_MESSAGE_PRO) {
		sprintf(tmpstr,"AT+MCT=%d+MCG=%d",s_tollsaver,s_loudness);
		sendstring(tmpstr,NULL, NULL);
	}
	writetolog("im on");
	return(0);
}
int repl_imon_proper(const char *s) {
	char tmpstr[256];
	if (!device_initialized) {
		logit(L_INFO,"Device not initialize - calling initdevice!\n");
		repl_initdevice(NULL);
	}
	setiospeed(B38400);
	sprintf(tmpstr,"AT+MCF=%d+MCV=%d+MCM=%d+MCD=%d+MCR=%d+MCP=%04d",
		s_fax,s_voice,s_monitor,s_remote,s_answer,s_password);
	sendstring(tmpstr, NULL,NULL);
	sprintf(tmpstr,"AT+MFI=\"%s\"",s_identity);
	sendstring(tmpstr, NULL,NULL);
	if (modemtypes[s_modemtype].id == USR_MESSAGE_PRO) {
		sprintf(tmpstr,"AT+MCT=%d+MCG=%d",s_tollsaver,s_loudness);
		sendstring(tmpstr,NULL, NULL);
	}
	/* reset clock but do NOT write libfile */
	resetclock(0);
	/* sendstring("AT&W", 500,NULL); */
	writetolog("im on");
	return(0);
}
int repl_imoff(const char *s) {
	if (!device_initialized) {
		logit(L_INFO,"Device not initialize - calling initdevice!\n");
		repl_initdevice(NULL);
	}
	setiospeed(B38400);
	sendstring("AT+MCS=0&W", NULL,NULL);
	writetolog("im off");
	repl_closedevice(NULL);
	return(0);
}
int repl_getall(const char *s) {
	if (!have_msginfos) {
		logit(L_INFO,"No msginfos - calling getmsginfo!\n");
		repl_getmsginfo(NULL);
	}
	if ( ( nr_voice + nr_fax ) > 0 ) {
		readclock();
		setiospeed(B115200);
		msgtofile();
	}
	writetolog("downloaded all messages");
	return(0);
}
int repl_getnew(const char *s) {
	if (!have_msginfos) {
		logit(L_INFO,"No msginfos - calling getmsginfo!\n");
		repl_getmsginfo(NULL);
	}
	newtofile();
	return(0);
}
int repl_getone(const char *s) {
	char *p;
	long int l;

	if (!have_msginfos) {
		logit(L_INFO,"No msginfos - calling getmsginfo!\n");
		repl_getmsginfo(NULL);
	}
	l = strtol(s,&p,10);
	if (p != NULL && ((l > 0) || (l <= ( nr_voice + nr_fax ))) ) {
		readclock();
		setiospeed(B115200);
		onetofile(l);
	} else {
		logit(L_ERROR,"getone: index out of range: %ld\n",l);
		return(1);
	}
	writetolog("downloaded one messages");
	return(0);
}
int repl_get(const char *s) {
	char *p;
	long int l;

	if (!have_msginfos) {
		logit(L_INFO,"No msginfos - calling getmsginfo!\n");
		repl_getmsginfo(NULL);
	}
	if (strcasecmp(s,"all") == 0) {
		if (nr_msgs == 0) {
			logit(L_INFO,"No messages!\n");
			return(0);
		}
		readclock();
		setiospeed(B115200);
		msgtofile();
		writetolog("downloaded all messages");
		return(0);
	}
	if (strcasecmp(s,"new") == 0) {
		if (nr_fax_new + nr_voice_new == 0) {
			logit(L_INFO,"No new messages!\n");
			return(0);
		}
		newtofile();
		writetolog("downloaded new messages");
		return(0);
	}
	if (strcasecmp(s,"greeting") == 0) {
		transfer_special(0);
		writetolog("downloaded greeting message");
		return(0);
	}
	if (strcasecmp(s,"full") == 0) {
		transfer_special(1);
		writetolog("downloaded full message");
		return(0);
	}
	l = strtol(s,&p,10);
	if (p != NULL && (l > 0) && (l <= nr_msgs) ) {
		readclock();
		setiospeed(B115200);
		onetofile(l);
	} else {
		logit(L_ERROR,"get: index out of range: %ld\n",l);
		return(1);
	}
	writetolog("downloaded one messages");
	return(0);
}
int repl_clear(const char *s) {
	if (!device_initialized) {
		logit(L_INFO,"Device not initialize - calling initdevice!\n");
		repl_initdevice(NULL);
	}
	logit(L_INFO,"Erasing modem memory... This takes about half a minute\n");
	setiospeed(B115200);
	sendstring("AT+MEM", NULL,NULL);
	writetolog("modem memory erased");
	/* Reseting clock AND (arg=1) write libfile! */
	resetclock(1);
        if (have_msginfos) {
                free(msginfos);
                have_msginfos = 0;
        }
	return(0);
}
int repl_ulg(const char *s) {
	modemval tmp;
	
	if (!device_initialized) {
		logit(L_INFO,"Device not initialize - calling initdevice!\n");
		repl_initdevice(NULL);
	}
	setiospeed(B38400); /* ???????????? */
	tmp=sendstring("AT+MVC=0",NULL,"CONNECT");
	if (tmp != M_OK) {
		logit(L_ERROR,"Cannot upload greeting file!\n");
		return(1);
	}
	ulfile(s);
	usleep(5000000);
	return(0);
}
int repl_ulf(const char *s) {
	modemval tmp;
	
	if (!device_initialized) {
		logit(L_INFO,"Device not initialize - calling initdevice!\n");
		repl_initdevice(NULL);
	}
	setiospeed(B38400); /* ???????????? */
	tmp = sendstring("AT+MVC=1",NULL,"CONNECT");
	if (tmp != M_OK) {
		logit(L_ERROR,"Cannot upload FullMessage file!\n");
		return(1);
	}
	ulfile(s);
	usleep(5000000);
	return(0);
}
int repl_recg(const char *s) {
	if (!device_initialized) {
		logit(L_INFO,"Device not initialize - calling initdevice!\n");
		repl_initdevice(NULL);
	}
	printf("Press Ctrl-C to stop recording.\n");
	writetolog("new greeting message recorded");
	setiospeed(B38400); /* ???????????? */
	sendstring("AT+MVR=0", NULL,NULL);
	return(0);
}
int repl_recf(const char *s) {
	if (!device_initialized) {
		logit(L_INFO,"Device not initialize - calling initdevice!\n");
		repl_initdevice(NULL);
	}
	printf("Press Ctrl-C to stop recording.\n");
	writetolog("new memory full message recorded");
	setiospeed(B38400); /* ???????????? */
	sendstring("AT+MVR=1", NULL,NULL);
	return(0);
}
int repl_view(const char *s) {
        long int l;
        char *p,str[5];

        if (!have_msginfos) {
                logit(L_INFO,"No msginfos - calling getmsginfo!\n");
                repl_getmsginfo(NULL);
        }
	if (strcasecmp(s,"all") == 0) {
		if (nr_fax == 0) {
			logit(L_INFO,"No fax messages!\n");
			return(0);
		}
		for (l=0;l<nr_msgs;l++) {
			if (is_fax(l)) {
				sprintf(str,"%3ld",l+1);
				repl_view(str);
			}
		}
		return(0);
	}
	if (strcasecmp(s,"new") == 0) {
		if (nr_fax_new == 0) {
			logit(L_INFO,"No new fax messages!\n");
			return(0);
		}
		for (l=0;l<nr_msgs;l++) {
			if (is_new_or_unknown(l) && is_fax(l)) {
				sprintf(str,"%3ld",l+1);
				repl_view(str);
			}
		}
		return(0);
	}
        l = strtol(s,&p,10);
        if (p == NULL || (l <= 0) || (l > ( nr_voice + nr_fax )) ) {
                logit(L_ERROR,"view: index out of range: %ld\n",l);
                return(1);
        }
        if (msginfos[l-1].type != T_FAX) {
                logit(L_ERROR,"view: can only view fax message!\n");
                return(1);
        }
	return view_fax(l);
}	
int repl_play(const char *s) {
	char tmpstr[256];
	long int l;
	char *p,str[5];
	
	if (!have_msginfos) {
		logit(L_INFO,"No msginfos - calling getmsginfo!\n");
		repl_getmsginfo(NULL);
	}
	if (strcasecmp(s,"all") == 0) {
		if (nr_voice == 0) {
			logit(L_INFO,"No voice messages!\n");
			return(0);
		}
		for (l=0;l<nr_msgs;l++) {
			if (is_voice(l)) {
				sprintf(str,"%3ld",l+1);
				repl_play(str);
			}
		}
		return(0);
	}
	if (strcasecmp(s,"new") == 0) {
		if (nr_voice_new == 0) {
			logit(L_INFO,"No new voice messages!\n");
			return(0);
		}
		for (l=0;l<nr_msgs;l++) {
			if (is_new_or_unknown(l) && is_voice(l)) {
				sprintf(str,"%3ld",l+1);
				repl_play(str);
			}
		}
		return(0);
	}
	if (strcasecmp(s,"greeting") == 0) {
		return repl_playg(NULL);
	}
	if (strcasecmp(s,"full") == 0 ) {
		return repl_playf(NULL);
	}
	l = strtol(s,&p,10);
	if (p == NULL || (l <= 0) || (l > ( nr_voice + nr_fax )) ) {
		logit(L_ERROR,"play: index out of range: %ld\n",l);
		return(1);
	}
	if (msginfos[l-1].type != T_VOICE) {
		logit(L_ERROR,"play: cannot only play voice message!\n");
		return(1);
	}
	if (s_out_card == 1) {
		return play_soundcard(l);
	}
	setiospeed(B115200);
	sprintf(tmpstr,"AT+MVP=%ld",l);
	/* install special signal handler */
	if (signal(SIGINT, interrupt_play_handler) == SIG_ERR) {
		logit(L_ERROR,"Couldn't install signal handler\n");
	}
	sendstring(tmpstr, NULL,NULL);
	/* restoring signal handler */
	signal(SIGINT, termination_handler);
	return(0);
}
int repl_playg(const char *s) {
	if (!device_initialized) {
		logit(L_INFO,"Device not initialize - calling initdevice!\n");
		repl_initdevice(NULL);
	}
	if (s_out_card == 1) {
		return play_soundcard(0);
	}
	setiospeed(B115200);
	/* install special signal handler */
	if (signal(SIGINT, interrupt_play_handler) == SIG_ERR) {
		logit(L_ERROR,"Couldn't install signal handler\n");
	}
	sendstring("AT+MVP=0", NULL,NULL);
	/* restoring signal handler */
	signal(SIGINT, termination_handler);
	return(0);
}
int repl_playf(const char *s) {
	if (!device_initialized) {
		logit(L_INFO,"Device not initialize - calling initdevice!\n");
		repl_initdevice(NULL);
	}
	if (s_out_card == 1) {
		return play_soundcard(255);
	}
	setiospeed(B115200);
	/* install special signal handler */
	if (signal(SIGINT, interrupt_play_handler) == SIG_ERR) {
		logit(L_ERROR,"Couldn't install signal handler\n");
	}
	sendstring("AT+MVP=255", NULL,NULL);
	/* restoring signal handler */
	signal(SIGINT, termination_handler);
	return(0);
}
int repl_list(const char *s) {
	if (!have_msginfos) {
		logit(L_INFO,"No msginfos - calling getmsginfo!\n");
		repl_getmsginfo(NULL);
	}
	setiospeed(B19200);
	printf("Number of new messages: voice %d fax %d\n",
	       nr_voice_new,nr_fax_new);
	printf("Number of total messages: voice %d fax %d\n",nr_voice,nr_fax);
	printf("Total memory: %dMb\n",total_memory);
	printf("Memory full up to %d%%!\n",percentfull);
	printf("Incoming message present: %s\n",inmsg?"yes":"no");
	printf("Full message present: %s\n",fullmsg?"yes":"no");
	listmsg();
	return(0);
}
int repl_report(const char *s) {
	if (!have_msginfos) {
		logit(L_INFO,"No msginfos - calling getmsginfo!\n");
		repl_getmsginfo(NULL);
	}
	setiospeed(B19200);
	printf("Number of new messages: voice %d fax %d\n",
	       nr_voice_new,nr_fax_new);
	printf("Number of total messages: voice %d fax %d\n",nr_voice,nr_fax);
	printf("Total memory: %dMb\n",total_memory);
	printf("Memory full up to %d%%!\n",percentfull);
	printf("Incoming message present: %s\n",inmsg?"yes":"no");
	printf("Full message present: %s\n",fullmsg?"yes":"no");
	report_msgs();
	return(0);
}
int repl_clock(const char *s) {
	if (!device_initialized) {
		logit(L_INFO,"Device not initialize - calling initdevice!\n");
		repl_initdevice(NULL);
	}
	setiospeed(B38400);
	readclock();
	return(0);
}
int repl_freenr(const char *s) {
	c_firstfree=freenr();
        logit(L_INFO,"First free message file number: %d\n",c_firstfree);
	return(0);
}
int repl_repl(const char *s) {
	return(start_repl());
}
int repl_quit(const char *s) {
	exit(0);
}
int repl_faxdir(const char *s) {
	strncpy(s_faxdir,s,128);
	return(0);
}
int repl_voicedir(const char *s) {
	strncpy(s_voicedir,s,128);
	c_firstfree = freenr();
	return(0);
}
int repl_fax(const char *s) {
	if (is_true(s)) s_fax = 1;
	else s_fax = 0;
	return(0);
}
int repl_voice(const char *s) {
	if (is_true(s)) s_voice = 1;
	else s_voice = 0;
	return(0);
}
int repl_monitor(const char *s) {
	if (is_true(s)) s_monitor = 1;
	else s_monitor = 0;
	return(0);
}
int repl_remote(const char *s) {
	if (is_true(s)) s_remote = 1;
	else s_remote = 0;
	return(0);
}
int repl_answer(const char *s) {
	char *p;
	long int l;

	l = strtol(s,&p,10);
	if (p == NULL || l < 3 || l > 6) {
		logit(L_ERROR,"syntax error: answer 3|4|5|6\n");
		return(1);
	} else {
		s_answer = l;
		return(0);
	}
}
int repl_set_recordtime(const char *s) {
	char *p;
	char tmpstr[256];
	long int l;

	l = strtol(s,&p,10);
	if (p == NULL || l < 0 || l > 255) {
		logit(L_ERROR,"syntax error: set_recordtime [0-255]\n");
		return(1);
	} else {
		if (!device_initialized) {
			logit(L_INFO,"Device not initialize - calling initdevice!\n");
			repl_initdevice(NULL);
		}
		setiospeed(B38400);
		sprintf(tmpstr,"AT+MVD=%d",(int)l);
		sendstring(tmpstr, NULL,NULL);
	}
	return(0);
}
int repl_password(const char *s) {
	char *p;
	long int l;
	l = strtol(s,&p,10);
	if (p == NULL || l < 999 || l > 9999) {
		logit(L_ERROR,"syntax error: password <4-digit-number>\n");
		return(1);
	} else {
		s_password = l;
		return(0);
	}
}
int repl_savepage(const char *s) {
	char *p;
	long int l;
	char fn[20];
	l = strtol(s,&p,10);
	if (p == NULL || l > 255 || l < 0) {
		logit(L_ERROR,"syntax error: savepage <page-number>\n");
		return(1);
	} else {
		sprintf(fn,"page-%d.data",(int)l);
		savepagetofile((int)l,fn);
		return(0);
	}
}
int repl_identity(const char *s) {
	strncpy(s_identity,s,21);
	return(0);
}
int repl_device(const char *s) {
	strncpy(s_device,s,32);
	return(0);
}
int repl_verbose(const char *s) {
	char *p;
	long int l;

	if (strcasecmp(s,"silent") == 0) {
		s_verbose = L_SILENT;
		return(0);
	}
	if (strcasecmp(s,"error") == 0) {
		s_verbose = L_ERROR;
		return(0);
	}
	if (strcasecmp(s,"info") == 0) {
		s_verbose = L_INFO;
		return(0);
	}
	if (strcasecmp(s,"debug") == 0) {
		s_verbose = L_DEBUG;
		return(0);
	}
	l = strtol(s,&p,10);
	if (p == NULL || l < 0 || l > 3) {
		logit(L_ERROR,"syntax error: verbose 0|1|2|3\n");
		return(1);
	} else {
		s_verbose = l;
		return(0);
	}
}
int repl_initstring(const char *s) {
	strncpy(s_init,s,32);
	return(0);
}
int repl_libfile(const char *s) {
	strncpy(s_libfile,s,256);
	return(0);
}
int repl_logfile(const char *s) {
	strncpy(s_logfile,s,256);
	return(0);
}
int repl_lockdir(const char *s) {
	strncpy(s_lockdir,s,256);
	return(0);
}
int repl_configfile(const char *s) {
	strncpy(s_configfile,s,256);
	return(0);
}
int repl_getmsginfo(const char *s) {
	if (!device_initialized) {
		logit(L_INFO,"Device not initialize - calling initdevice!\n");
		repl_initdevice(NULL);
	}
	setiospeed(B19200);
	getmsginfo();
	readclock();
	have_msginfos = 1;
	return(0);
}
int repl_parseconfig(const char *s) {
	return(parseconfigfile());
}
int repl_initdevice(const char *s) {
	char tmpstr[128];
	
	opendev();
	sprintf(tmpstr,"%s",s_init);
	if ( sendstring(tmpstr, NULL, NULL) == M_NORESPONSE) {
		logit(L_ERROR,"Modem not responding\n");
		exit(1);
	}
	if ( sendstring("AT+MCS=1", NULL, NULL) == M_NORESPONSE) {
		logit(L_ERROR,"Modem not responding\n");
		exit(1);
	}
	device_initialized = 1;
	return(0);
}
int repl_closedevice(const char *s) {
/*	if (!device_initialized) {
		return(0);
	} */
	/* Call closedev in any case, because lock_file MUST be removed! */
	closedev();
	if (have_msginfos) {
		free(msginfos);
		have_msginfos = 0;
	}
	device_initialized = 0;
	return(0);
}
int repl_help(const char *s) {
	Tcomdef *cd = &comdef[0];
	
	printf("smmgr %s\n%s\nUsage:\nsmmgr [noconfig] [<commands> ...]\nCommands:\n",
	       VERSION, COPYRIGHT);
	while (cd->name != NULL) {
		printf("\t%-15s%-5s: %-30s\n",cd->name,(cd->takearg)?"arg":"",
		       cd->description);
		cd++;
	}
	return(0);
}
int repl_settings(const char *s) {
	printf("smmgr: current settings:\n");
	printf("ModemType: %s\n",modemtypes[s_modemtype].modemname);
	printf("s_fax=%d\ns_voice=%d\ns_monitor=%d\ns_remote=%d\n",
	       s_fax,s_voice,s_monitor,s_remote);
	printf("s_answer=%d\ns_verbose=%d\ns_identity=%s\ns_device=%s\n",
	       s_answer,s_verbose,s_identity,s_device);
	printf("s_init=%s\ns_faxdir=%s\ns_voicedir=%s\ns_libfile=%s\n",
	       s_init,s_faxdir,s_voicedir,s_libfile);
	printf("s_logfile=%s\ns_lockdir=%s\ns_configfile=%s\n",
	       s_logfile,s_lockdir,s_configfile);
	printf("s_ignore_dle=%d\ns_imon_on_exit=%d\n",
	       s_ignore_dle,s_imon_on_exit);
	printf("s_mail_on_download=%d\ns_mailprog=%s\ns_faxmail=%s\ns_voicemail=%s\n",
	       s_mail_on_download,s_mailprog, s_faxmail, s_voicemail);
	printf("s_tollsaver=%d\ns_loudness=%d\n",s_tollsaver,s_loudness);
	printf("s_viewprog=%s\ns_localscript=%s\n",s_viewprog,s_localscript);
	return(0);
}
int repl_pagesingle(const char *s) {
	logit(L_INFO,"Don't use at the moment - buggy! returning ...\n");
	return(0);
	if (is_true(s)) s_pagesingle = 1;
	else s_pagesingle = 0;
	return(0);
}
int repl_ignore_dle(const char *s) {
	if (is_true(s)) s_ignore_dle = 1;
	else s_ignore_dle = 0;
	return(0);
}
int repl_imon_on_exit(const char *s) {
	if (is_true(s)) s_imon_on_exit = 1;
	else s_imon_on_exit = 0;
	return(0);
}
int repl_modemstatus(const char *s) {
	setiospeed(B19200);
	getmodemstatus();
	printf("Current modem status:\n");
	printf("%-19s | %3s | %-15s | %s\n","Option","CMD","command","value");
	printf("--------------------|-----|-----------------|------------\n");
	printf("%-19s | %3s | %-15s | %d\n","SelfMode","MCS","",modemstat.mcs);
	printf("%-19s | %3s | %-15s | %d\n",
	       "Dialup Retrieval","MCD","remote",modemstat.mcd);
	printf("%-19s | %3s | %-15s | %d\n",
	       "Fax receive cap.","MCF","fax",modemstat.mcf);
	printf("%-19s | %3s | %-15s | %d\n",
	       "Voice receive cap.","MCV","voice",modemstat.mcv);
	printf("%-19s | %3s | %-15s | %d\n",
	       "Local Mode (??)","MCL","",modemstat.mcl);
	printf("%-19s | %3s | %-15s | %d\n",
	       "Monitor","MCM","monitor",modemstat.mcm);
	printf("%-19s | %3s | %-15s | %d\n",
	       "Password","MCP","password",modemstat.mcp);
	printf("%-19s | %3s | %-15s | %d\n",
	       "Number of Rings","MCR","answer",modemstat.mcr);
	printf("%-19s | %3s | %-15s | %d\n",
	       "Record time","MVD","set_recordtime",modemstat.mvd);
	printf("%-19s | %3s | %-15s | %s\n",
	       "Clock","MCC","",modemstat.mcc);
	printf("%-19s | %3s | %-15s | %s\n",
	       "Identity","MFI","identity",modemstat.mfi);
	if (modemtypes[s_modemtype].id == USR_MESSAGE_PRO) {
		printf("%-19s | %3s | %-15s | %d\n",
		       "Tollsaver", "MCT", "tollsaver",modemstat.mct);
		printf("%-19s | %3s | %-15s | %d\n",
		       "Loudness", "MCG", "loudness",modemstat.mcg);
	}
	return(0);
}
int repl_mail_on_download(const char *s) {
	if (is_true(s)) s_mail_on_download = 1;
	else s_mail_on_download = 0;
	return(0);
}
int repl_faxmail(const char *s) {
	strncpy(s_faxmail,s,256);
	return(0);
}
int repl_out_card(const char *s) {
	if (is_true(s)) s_out_card = 1;
	else s_out_card = 0;
	return(0);
}
int repl_voicemail(const char *s) {
	strncpy(s_voicemail,s,256);
	return(0);
}
int repl_mailprog(const char *s) {
	strncpy(s_mailprog,s,256);
	return(0);
}
int repl_viewprog(const char *s) {
	strncpy(s_viewprog,s,256);
	return(0);
}
int repl_version(const char *s) {
	printf("smmgr %s\n%s\n",VERSION,COPYRIGHT);
	printf("SelfMode Manager, a program to manage a USR MessagePlus, Message56k and similar!\n");
	return(0);
}
int repl_localscript(const char *s) {
	strncpy(s_localscript,s,256);
	return(0);
}
int repl_editlocal(const char *s) {
	char ss[512];
	strcpy(ss,s_localscript);
	strcat(ss," ");
	strcat(ss,s_voicedir);
	strcat(ss," ");
	strcat(ss,s_faxdir);
	return(system(ss));
}
int repl_command(const char *s) {
	char tmpstr[128];
	char *ret=NULL;
	modemval r;
	if (!device_initialized) {
		logit(L_INFO,"Device not initialize - calling initdevice!\n");
		repl_initdevice(NULL);
	}
	setiospeed(B38400);
	strcpy(tmpstr,s);
	r = sendstring(tmpstr,&ret,NULL);
	if (r == M_ERROR) {
		logit(L_ERROR,"Got ERROR from modem!\n");
	}
	printf("%s\n",ret);
	free(ret);
	return(0);
}
int repl_modemtype(const char* s) {
        int n=0;
        while (modemtypes[n].modemname != NULL ) {
                if (strcasecmp( s, modemtypes[n].modemname ) == 0) {
                        s_modemtype=n;
                        return(0);
                }
                n++;
        }
        logit(L_INFO,"Modem not found: %s\n",s);
        return(P_NOTFOUND);
}
int repl_tollsaver(const char* s) {
	if (is_true(s)) s_tollsaver = 1;
	else s_tollsaver = 0;
	return(0);
}
int repl_loudness(const char* s) {
	char *p;
	long int l;
	l = strtol(s,&p,10);
	if (p == NULL || l < 0 || l > 255) {
		logit(L_ERROR,"syntax error: loudness <0-255>\n");
		return(1);
	} else {
		s_loudness = l;
		return(0);
	}
}
#ifdef MACRO
/* macro definitions:
   macro bla=getmsginfos
   macro "doallthejunk=getmsginfos list get all ..."
*/
int repl_macro(const char* s) {
	Tmacdef *nm,*mm;
	char data[256];
	char macname[256];
	char* t;
	int found=0;
	strcpy(data,s);
	t=&data[0];
	strcpy(macname,strsep(&t,"="));
	if ((macname == NULL) || (t == NULL)) {
		logit(L_ERROR,"syntax error: macro <macname>=<macdef>\n");
		return(1);
	}
	logit(L_DEBUG,"macname=%s, macexp=%s\n",macname,t);
	mm=macdef;
	while(mm->name != NULL) {
		logit(L_DEBUG,"We are in the search loop!\n");
		if (strcmp(mm->name,macname) == 0) {
			logit(L_INFO,"already found a definition for macro %s, replacing!\n",macname);
			found = 1; nm=mm;
			break;
		}
		mm=mm->next;
	}
	if (found == 0) {
		logit(L_DEBUG,"new definition: allocating space!\n");
		fflush(stderr); fflush(stdout);
		nm = (Tmacdef*)malloc(sizeof(Tmacdef));
		nm->next = macdef;
		macdef = nm;
		nm->name = (char*)malloc(strlen(macname)); 
		nm->expansion = (char*)malloc(strlen(t));
	} else {
		logit(L_DEBUG,"old definition: %s=%s\n",nm->name,nm->expansion);
		fflush(stderr); fflush(stdout);
		nm->name = (char*)realloc(nm->name,strlen(macname));
		nm->expansion= (char*)realloc(nm->expansion,strlen(t));
	}
	strcpy(nm->name, macname);
	strcpy(nm->expansion,t);
	return(0);
}
int repl_showmacros(const char* s) {
	Tmacdef *nm=macdef;
	if (nm->name == (char*)NULL) {
		printf("No macros defined!\n");
		return(0);
	}
	while (nm->name != (char*)NULL) {
		printf("%-10s  -->  %-50s\n",nm->name, nm->expansion);
		nm = nm->next;
	}
	return(0);
}
#endif
