/* 
  Copyright (C) 1998 Norbert Preining and Christofer Edvardsen

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "smmgr.h"

int is_not_comment(const char *s) {
	const char *p = s;
	while (*p == ' ' || *p == '\t') p++;
	if (*p == '#' || *p == '\n' || *p == '\0') return(0);
	return(1);
}

int count_words(const char *s) {
	int num = 0;
	const char *p = s;
	int inword = 0;
	
	while (1) {
		while (isblank(*p) || *p == '\n' || *p == '\0') {
			inword = 0;
			p++;
			if (*p == '\0' || *p == '\n') return(num);
		}
		while (!isblank(*p) && *p != '\n' && *p != '\0') {
			if (*p == '"') {
				p++;
				if (!inword) {
					num++;
					inword = 1;
				}
				if (*p == '\0') {
					logit(L_ERROR,"4- Syntax Error in configfile! Unbalanced quotes\n");
					return(-1);
				}
				while (*p != '"') {
					p++;
					if (*p == '\0') {
						logit(L_ERROR,"3- Syntax Error in configfile! Unbalanced quotes\n");
						return(-1);
					}
				}
				inword=0;
				p++;
			} else {
				p++;
				if (!inword) {
					num++;
					inword = 1;
				}
				if (*p == '\0') return(num);
			}
		}
	}
}

int line_to_array(char* s, char ***ar) {
	/* stupid done by me */
	char *p = s, **ap;
	int num = 0, inword = 0;

	/* first count */
	num = count_words(s);
	if (num < 0) return(num);
	*ar = malloc (num * sizeof(char*));
	ap = *ar;

	while (1) {
		while (isblank(*p) || *p == '\n') {
			if (inword) {
				*p = '\0';
				inword = 0;
			}
			p++;
			if (*p == '\0' || *p == '\n') return(num);
		}
		while (!isblank(*p) && *p != '\n') {
			if (*p == '"') {
				p++;
				if (!inword) {
					*ap = p;
					ap++;
					inword = 1;
				}
				if (*p == '\0') {
					logit(L_ERROR,"1- Syntax Error in configfile! Unbalanced quotes\n");
					return(-1);
				}
				
				while (*p != '"') {
					p++;
					if (*p == '\0') {
						logit(L_ERROR,"2- Syntax Error in configfile! Unbalanced quotes\n");
						return(-1);
					}
				}
				*p = '\0';
				inword = 0;
				p++;
			} else {
				p++;
				if (!inword) {
					*ap = (p-1);
					ap++;
					inword = 1;
				}
				if (*p == '\n') {
					*p = '\0';
					return(num);
				}
				if (*p == '\0') return(num);
			}
		}
	}
}

/*
  Convert the first n bytes of str to a number
*/

int str2num (char *s, int n) 
{
  int result=0;
  int base=1;
  for (;--n>=0;) {
	  result += (s[n]-'0')*base;
	  base *= 10;
  }
  return(result);
}


int attr2str(int t, char *s) 
{
      switch(t) {
      case 255: strncpy(s,"ur, uc",100);
	break;
      case 253: strncpy(s,"ur, c",100);
	break;
      case 252: strncpy(s,"r, c",100);
	break;
      default:  strncpy(s,"unknown",100);
      }
      return(0);
}

int is_true(const char* s) {
	if (strcasecmp(s,"true") == 0 ||
	    strcasecmp(s,"on") == 0 ||
	    strcasecmp(s,"yes") == 0 ||
	    strcasecmp(s,"1") == 0) return(1);
	return(0);
}

int is_new_or_unknown(int i) {
	if (msginfos[i].attr != 252) return(1);
	else return(0);
}

int is_voice(int i) {
	if (msginfos[i].type == T_VOICE) return(1);
	else return(0);
}

int is_fax(int i) {
	if (msginfos[i].type == T_FAX) return(1);
	else return(0);
}

int make_rtime(time_t rtime_day, time_t rtime_hour, time_t rtime_min,
		 char *rtime_short, char *rtime_str) {
	time_t rtime;
	struct tm *tmptime;
	
	get_stored_time();
	if ( (rtime_hour>60) || (rtime_min>60) ) {
		strcpy(rtime_short,"unknown");
		strcpy(rtime_str,"unknown");
	} else {
		rtime=stored_time+rtime_day*3600*24+rtime_hour*3600+rtime_min*60;
		tmptime=localtime(&rtime);
		strcpy(rtime_str,asctime(tmptime));
		rtime_str[strlen(rtime_str)-1]=0;
		strftime(rtime_short,32," %a %b %d %H:%M",tmptime);
	}
	return(0);
}

int make_commentfile(char *commentname, int c_index,
		 char *rtime_short, char *rtime_str, char *title) {
	FILE *commentfile;
	int flag=0;
	
	commentfile = fopen(commentname,"w");
	if ( commentfile == NULL ) {
		printf("Failed to open file for output: %s\n",commentname);
		return(-1);
	}
	fprintf(commentfile,"Title: %s\n",title);
	if ( msginfos[c_index-1].type == 1 ) {
		fprintf(commentfile,"Sender FaxId: %s\n",
			msginfos[c_index-1].id);
		fprintf(commentfile,"Pages: %d\n",
			msginfos[c_index-1].pagetime);
	} else {
		fprintf(commentfile,"Length: %d s\n",
			msginfos[c_index-1].pagetime);
	}
	fprintf(commentfile,"ReceiveTime: %s\n",rtime_str);
	if ( ( timedrift > 3600 ) || ( timedrift < -3600 ) ) {
		fprintf(commentfile,
			"Warning: Receive time might be unreliable\n");
		flag=1;
	}
	fprintf(commentfile,"DataString#%s#%s#%d#%s#%d\n",
			title,msginfos[c_index-1].id,msginfos[c_index-1].pagetime,rtime_str,flag);
	fclose(commentfile);
	return(0);
}


int make_names(char *msgname, char *commentname, char *title,
		int c_index, char *rtime_short) {
	int ff=c_firstfree==-1?freenr():c_firstfree;
	switch (msginfos[c_index-1].type) {
	case T_FAX:
		sprintf(msgname,"%s/%s%04d",s_faxdir,s_faxprefix,ff);
		sprintf(commentname,"%s/%s%04d.txt",s_faxdir,s_faxprefix,ff);
		sprintf(title,"Fax %dp %s %s",
			msginfos[c_index-1].pagetime,
			msginfos[c_index-1].id,
			rtime_short);
		break;
	case T_VOICE:
		sprintf(msgname,"%s/voice%04d.gsm",s_voicedir,ff);
		sprintf(commentname,"%s/voice%04d.txt",s_voicedir,ff);
		sprintf(title,"Voice %ds %s",
			msginfos[c_index-1].pagetime,
			rtime_short);
		break;
	default:
		sprintf(msgname,"%s/unknown%04d.%d",s_faxdir,ff,
			msginfos[c_index-1].type);
		sprintf(commentname,"%s/unknown%04d.txt",s_faxdir,ff);
		strcpy(title,"Unknown");
	}
	return(0);
}

	
int transfer_no_dle_handle(int n_page, int n_offset, char *msgname, 
			    int c_msg, char *commentname) {
	FILE *msgfile;

	printf("entering transfer no dle\n");
	msgfile = fopen(msgname,"w");
	if ( msgfile == NULL ) {
		printf("Failed to open messagefile for output: %s\n",msgname); 
		return(-1);
	} else {
		logit(L_INFO,"Writing message %d to %s\n",c_msg,msgname);
	}
	while ( (c_page < n_page) || (c_offset < n_offset) ) {
		fwrite(&page[c_offset],1,1,msgfile);
		incoffset(1);
	}
	fclose(msgfile);
	return(0);
}

int transfer_fax(int n_page, int n_offset, char *bname, 
		  int c_msg, char* commentname) {
	int prev_dle = 0;
	int faxpage = 1;
	char msgname[256];
	FILE* msgfile;
	char **fnamelist;
	int i, ret = 0;
	
	fnamelist = (char**)malloc(msginfos[c_msg].pagetime*sizeof(char*));
	
	if (s_pagesingle == 1) {
		sprintf(msgname,"%sp%03d.g3",bname,faxpage);
	} else {
		sprintf(msgname,"%s.g3",bname);
	}
	msgfile = fopen(msgname,"w");
	if ( msgfile == NULL ) {
		printf("Failed to open messagefile for output: %s\n",msgname); 
		return(-1);
	} else {
		logit(L_INFO,"Writing message %d to %s\n",c_msg,msgname);
	}
	fnamelist[faxpage-1] = (char*)malloc(256*sizeof(char));
	strncpy(fnamelist[faxpage-1],msgname,256);
	while ( (c_page < n_page) || (c_offset + 1 < n_offset) ) {
		logit(L_DEBUG,"transfer_fax: while_loop: c_page=%d,n_page=%d,coffset=%6d,n_offset=%6d\n",c_page,n_page,c_offset,n_offset);
		if ( prev_dle == 1 ) {
			switch (page[c_offset]) {
			case 0x3:
				if (s_pagesingle == 1) {
					fclose(msgfile);
					faxpage++;
					sprintf(msgname,"%sp%03d.g3",bname,faxpage);
					msgfile = fopen(msgname,"w");
					if ( msgfile == NULL ) {
						printf("Failed to open messagefile for output: %s\n",msgname); 
						return(-1);
					} else {
						logit(L_INFO,"Writing message %d page %d to %s\n",c_msg,faxpage,msgname);
					}
					fnamelist[faxpage-1] = (char*)malloc(256*sizeof(char));
					strncpy(fnamelist[faxpage-1],msgname,256);
				} else {
					fwrite(&page[c_offset-1],1,1,msgfile);
					fwrite(&page[c_offset],1,1,msgfile);
				}
				break;
			case 0x10:
				fwrite(&page[c_offset],1,1,msgfile);
				break;
			case 0x18:
				logit(L_DEBUG,"transfer: Got <DLE><CAN>, looks like end ;-)\n");
				break;
			default:
				logit(L_ERROR,"After <DLE>: 0x%x\n",page[c_offset]);
				logit(L_DEBUG,"page=%d, offset=%d\n",c_page,c_offset);
			}
			prev_dle = 0;
		} else {
			if ( page[c_offset] == 0x10 ) {
				prev_dle = 1;
			} else {
				fwrite(&page[c_offset],1,1,msgfile);
			}
		}
		incoffset(1);
	}
	fclose(msgfile);
	if (s_mail_on_download) {
		ret = send_files(commentname,fnamelist,
				 msginfos[c_msg].pagetime, T_FAX);
	}
	for (i=0; i<faxpage; i++) free(fnamelist[i]);
	free(fnamelist);
	return (ret);
}


int transfer_voice(int n_page, int n_offset, char *msgname, 
		    int c_msg, char *commentname) {
	int prev_dle = 0;
	FILE *msgfile;
	char **fnamelist;
	int ret=0;

	msgfile = fopen(msgname,"w");
	if ( msgfile == NULL ) {
		printf("Failed to open messagefile for output: %s\n",msgname); 
		return(-1);
	} else {
		logit(L_INFO,"Writing message %d to %s\n",c_msg,msgname);
	}
	while ( (c_page < n_page) || (c_offset < n_offset) ) {
		if ( prev_dle == 1 ) {
			switch (page[c_offset]) {
			case 0x10:
				fwrite(&page[c_offset],1,1,msgfile);
				break;
			case 0x18:
				logit(L_DEBUG,"transfer: Got <DLE><CAN>, looks like end ;-)\n");
				break;
			default:
				logit(L_ERROR,"After <DLE>: 0x%x\n",page[c_offset]);
				logit(L_DEBUG,"page=%d, offset=%d\n",c_page,c_offset);
			}
			prev_dle = 0;
		} else {
			if ( page[c_offset] == 0x10 ) {
				prev_dle = 1;
			} else {
				fwrite(&page[c_offset],1,1,msgfile);
			}
		}
		incoffset(1);
	}
	fclose(msgfile);
	if (s_mail_on_download) {
		fnamelist = (char**)malloc(sizeof(char*));
		fnamelist[0] = (char*)malloc(256*(sizeof(char)));
		strncpy(fnamelist[0],msgname,256);
		ret = send_files(commentname,fnamelist,1, T_VOICE);
		free(fnamelist[0]);
		free(fnamelist);
	}
	return(ret);
}

/* 
   transfer_special
   transfers full or greeting message
*/
int transfer_special(int n) {
	int prev_dle = 0;
	FILE *msgfile;
	int done = 0;
	char fname[256];

	strncpy(fname,s_voicedir,256);
	if (n==0) strncat(fname,"/greeting-msg.gsm",256);
	else strncat(fname,"/full-msg.gsm",256);
	msgfile = fopen(fname,"w");
	if ( msgfile == NULL ) {
		logit(L_ERROR,"Failed to open messagefile for output: %s\n",fname); 
		return(-1);
	} else {
		logit(L_INFO,"Writing %s message to %s\n",n==0?"greeting":"full",fname);
	}
	get_page(n);
	c_offset = 0;
	incoffset(33);
	while (done == 0) {
		if (c_page != n) {
			logit(L_ERROR,"Ooops: found no end in page %d, terminating download!\n",n);
			fclose(msgfile);
			return(1);
		}
		if ( prev_dle == 1 ) {
			switch (page[c_offset]) {
			case 0x10:
				fwrite(&page[c_offset],1,1,msgfile);
				break;
			case 0x18:
			case 0x03:
				logit(L_DEBUG,"transfer: Got <DLE><CAN> or <DLE><ETX>, looks like end ;-)\n");
				done = 1;
				break;
			default:
				logit(L_ERROR,"After <DLE>: 0x%x\n",page[c_offset]);
				logit(L_DEBUG,"page=%d, offset=%d\n",c_page,c_offset);
			}
			prev_dle = 0;
		} else {
			if ( page[c_offset] == 0x10 ) {
				prev_dle = 1;
			} else {
				fwrite(&page[c_offset],1,1,msgfile);
			}
		}
		incoffset(1);
	}
	fclose(msgfile);
	return(0);
}

int send_files( char *commentname, char **fnamelist, int len, int type) {
	int i,kl;
	char sys[512],tmpstr[20];
	char *p, *rcpt;

/* mailprog string:
   %f       ->     datafile1 datafile2 ...
   %str.F   ->     str datafile1 str datafile2 ...
   %r       ->     recipient
   %d       ->     infofile
   %s       ->     subject
*/
	rcpt = (type==T_FAX?s_faxmail:s_voicemail);
	p = s_mailprog;
	strcpy(sys,"");
	strcpy(tmpstr,"");
	kl = 0;
	while (*p != '\0') {
		if (*p != '%') {
			strncat(sys,p,1);
		} else {
			p++;
			switch (*p) {
			case 'r': strcat(sys,rcpt);
				strcat(sys," ");
				break;
			case 'd': strcat(sys,commentname);
				strcat(sys," ");
				break;
			case 's': strcat(sys,"\"New Fax/Voice msg\"");
				strcat(sys," ");
				break;
			case 'f': for (i=0; i<len; i++) {
				strcat(sys,fnamelist[i]);
				strcat(sys," ");
			}
			break;
			case '(': p++;
				while (*p != ')' || kl > 0) {
					strncat(tmpstr,p,1);
					if (*p == '(') kl++;
					if (*p == ')') kl--;
					p++;
				}
				p++;
				if (*p != 'F') {
					logit(L_ERROR,"Error Format: %%(str)F\n");
					return(-1);
				}
				for (i=0; i<len; i++) {
					strcat(sys,tmpstr);
					strcat(sys,fnamelist[i]);
					strcat(sys," ");
				}
				break;
			default: logit(L_ERROR,"Error Format: %%f, %%(str)F, %%r, %%d, %%s recognized!\n");
				return(-1);
				break;
			}
		}
		p++;
	}
	return(system(sys));
}

char *dupstr (const char *s) {
	char *r;
	r = xmalloc (strlen (s) + 1);
	strcpy (r, s);
	return (r);
}

void logit(logval l,const char *fmt, ...) {
	va_list ap;
	va_start(ap,fmt);
	if (s_verbose >= l)
		vfprintf(stderr,fmt,ap);		
	va_end(ap);
}

int get_stored_time() {
	FILE *timefile;
	if (stored_time != -1) return(0);
        timefile=fopen(s_libfile,"r");
        if (timefile==NULL) {
                logit(L_ERROR,"Couldn't read %s\n",s_libfile);
                logit(L_ERROR,"Will not be able to determine receive time of messages\n");
                return(1);
        }
        fscanf(timefile,"%d\n", (int*)&stored_time);
        fclose(timefile);
	return(0);
}

int set_stored_time(time_t t) {
	FILE *timefile;
        timefile=fopen(s_libfile,"w");
        if (timefile==NULL) {
                logit(L_ERROR,"Couldn't write the time to %s\n",s_libfile);
                logit(L_ERROR,"Will not be able to determine receive time of messages\n");
                return(1);
        }
        fprintf(timefile,"%d\n",(int)t);
	fclose(timefile);
	stored_time = t;
	return(0);
}

